@@ -39,9 +39,11 @@
 #include "otp.h"
 #include "add_window.h"
 #include "gram.tab.h"
+#include "vscreen.h"
 #include "win_decorations.h"
 #include "win_resize.h"
 #include "win_utils.h"
+#include "xparsegeometry.h"
 
 
 /* Where we start drawing the name in the icon manager */
@@ -89,6 +91,11 @@ void CreateIconManagers(void)
 		Scr->siconifyPm = Create2DIconManagerIcon();
 	}
 
+	// This loop is confusing.  The inner for() loops p over the ->next
+	// elements in the list, which is all the iconmgr's in the workspace.
+	// The outer for() loops q over the ->nextv (<-- extra 'v' on the
+	// end), which is a link to the head of the iconmgr list for the
+	// _next_ workspace.
 	ws = Scr->workSpaceMgr.workSpaceList;
 	for(IconMgr *q = Scr->iconmgr; q != NULL; q = q->nextv) {
 		for(IconMgr *p = q; p != NULL; p = p->next) {
@@ -104,8 +111,9 @@ void CreateIconManagers(void)
 			if(!p->geometry || !strlen(p->geometry)) {
 				p->geometry = "+0+0";
 			}
-			mask = XParseGeometry(p->geometry, &gx, &gy,
-			                      (unsigned int *) &p->width, (unsigned int *)&p->height);
+			mask = RLayoutXParseGeometry(Scr->Layout, p->geometry,
+			                             &gx, &gy,
+			                             (unsigned int *) &p->width, (unsigned int *)&p->height);
 
 			bw = LookInList(Scr->NoBorder, imname, NULL) ? 0 :
 			     (Scr->ThreeDBorderWidth ? Scr->ThreeDBorderWidth : Scr->BorderWidth);
@@ -165,28 +173,30 @@ void CreateIconManagers(void)
 
 
 			p->twm_win = AddWindow(p->w, AWT_ICON_MANAGER, p, Scr->currentvs);
-			/*
-			 * SetupOccupation() called from AddWindow() doesn't setup
-			 * occupation for icon managers, nor clear vs if occupation lacks.
-			 *
-			 * There is no Scr->currentvs->wsw->currentwspc set up this
-			 * early, so we can't check with that; the best check we can do
-			 * is use ws->number.  This may be incorrect when re-starting
-			 * ctwm.
-			 */
+
+			// SetupOccupation() called from AddWindow() doesn't setup
+			// occupation for icon managers, nor clear vs if occupation
+			// lacks.  So make it occupy the one we're setting up, or the
+			// 1st if we ran out somehow...
 			if(ws) {
 				p->twm_win->occupation = 1 << ws->number;
-				if(ws->number > 0) {
+
+				// ConfigureWorkSpaceManager() ran before us, so we can
+				// tell whether we're in the ws to reveal this IM.
+				if(ws->number != Scr->currentvs->wsw->currentwspc->number) {
 					p->twm_win->vs = NULL;
 				}
 			}
 			else {
 				p->twm_win->occupation = 1;
 			}
+
 #ifdef DEBUG_ICONMGR
 			fprintf(stderr,
-			        "CreateIconManagers: IconMgr %p: x=%d y=%d w=%d h=%d occupation=%x\n",
-			        p, gx, gy,  p->width, p->height, p->twm_win->occupation);
+			        "CreateIconManagers: IconMgr %p: twm_win=%p win=0x%lx "
+			        "name='%s' x=%d y=%d w=%d h=%d occupation=%x\n",
+			        p, p->twm_win, p->twm_win->w, p->name,
+			        gx, gy,  p->width, p->height, p->twm_win->occupation);
 #endif
 
 			{
@@ -1196,8 +1206,8 @@ void PackIconManager(IconMgr *ip)
 
 	XResizeWindow(dpy, ip->w, newwidth, ip->height);
 
-	mask = XParseGeometry(ip->geometry, &JunkX, &JunkY,
-	                      &JunkWidth, &JunkHeight);
+	mask = RLayoutXParseGeometry(Scr->Layout, ip->geometry, &JunkX, &JunkY,
+	                             &JunkWidth, &JunkHeight);
 	if(mask & XNegative) {
 		ip->twm_win->frame_x += ip->twm_win->frame_width - newwidth -
 		                        2 * ip->twm_win->frame_bw3D;
@@ -1247,8 +1257,7 @@ DrawIconManagerIconName(TwmWindow *tmp_win)
 		PackIconManagers();
 	}
 
-	DrawIconManagerBorder(iconmanagerlist, true);
-
+	// Write in the title
 	FB(iconmanagerlist->cp.fore, iconmanagerlist->cp.back);
 
 	/* XXX This is a completely absurd way of writing this */
@@ -1265,6 +1274,13 @@ DrawIconManagerIconName(TwmWindow *tmp_win)
 	 + ICON_MGR_IBORDER,
 	 tmp_win->icon_name,
 	 strlen(tmp_win->icon_name));
+
+	// Draw the border around it.  Our "border" isn't an X border, it's
+	// just our own drawing inside the X window.  Since XmbDrawString()
+	// believes it has all the space in the window to fill, it might
+	// scribble into the space where we're drawing the border, so draw
+	// the border after the text to cover it up.
+	DrawIconManagerBorder(iconmanagerlist, false);
 }
 
 
