@@ -202,11 +202,12 @@ SetupOccupation(TwmWindow *twm_win, int occupation_hint)
 	 * If it could be shown in one of the other vscreens, change the vscreen.
 	 */
 	if(!OCCUPY(twm_win, twm_win->vs->wsw->currentwspc)) {
-		VirtualScreen *vs;
 
 		twm_win->vs = NULL;
 
+#ifdef VSCREEN
 		if(Scr->numVscreens > 1) {
+			VirtualScreen *vs;
 			for(vs = Scr->vScreenList; vs != NULL; vs = vs->next) {
 				if(OCCUPY(twm_win, vs->wsw->currentwspc)) {
 					twm_win->vs = vs;
@@ -215,6 +216,7 @@ SetupOccupation(TwmWindow *twm_win, int occupation_hint)
 				}
 			}
 		}
+#endif
 	}
 
 
@@ -1072,7 +1074,6 @@ Occupy(TwmWindow *twm_win)
 {
 	int          x, y;
 	unsigned int width, height;
-	int          xoffset, yoffset;
 	Window       w;
 	struct OccupyWindow    *occupyWindow;
 	TwmWindow *occupy_twm;
@@ -1092,28 +1093,18 @@ Occupy(TwmWindow *twm_win)
 	             &JunkBW, &JunkDepth);
 	XQueryPointer(dpy, Scr->Root, &JunkRoot, &JunkRoot, &JunkX, &JunkY,
 	              &x, &y, &JunkMask);
-	x -= (width  / 2);
-	y -= (height / 2);
-	if(x < 0) {
-		x = 0;
-	}
-	if(y < 0) {
-		y = 0;
-	}
-	xoffset = width  + 2 * Scr->BorderWidth;
-	yoffset = height + 2 * Scr->BorderWidth + Scr->TitleHeight;
-
-	/* ... (but not off the screen!) */
-	if((x + xoffset) > Scr->rootw) {
-		x = Scr->rootw - xoffset;
-	}
-	if((y + yoffset) > Scr->rooth) {
-		y = Scr->rooth - yoffset;
-	}
 
 	occupy_twm = occupyWindow->twm_win;
 	occupy_twm->occupation = twm_win->occupation;
 
+	width += 2 * (occupy_twm->frame_bw3D + occupy_twm->frame_bw);
+	height += 2 * (occupy_twm->frame_bw3D + occupy_twm->frame_bw);
+	x -= (width  / 2);
+	y -= (height / 2);
+
+	/* Clip to screen */
+	ConstrainByLayout(Scr->BorderedLayout, -1, &x, width, &y, height);
+
 	/* Move the occupy window to where it should be */
 	if(occupy_twm->parent_vs != twm_win->parent_vs) {
 		occupy_twm->vs = twm_win->parent_vs;
